# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...requests.webhook_subscription import WebhookSubscriptionParams
from ...core.request_options import RequestOptions
from ...core.http_response import HttpResponse
from ...types.create_webhook_subscription_response import CreateWebhookSubscriptionResponse
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...types.get_webhook_subscription_response import GetWebhookSubscriptionResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...types.update_webhook_subscription_response import UpdateWebhookSubscriptionResponse
from ...types.delete_webhook_subscription_response import DeleteWebhookSubscriptionResponse
from ...types.update_webhook_subscription_signature_key_response import UpdateWebhookSubscriptionSignatureKeyResponse
from ...types.test_webhook_subscription_response import TestWebhookSubscriptionResponse
from ...core.client_wrapper import AsyncClientWrapper
from ...core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawSubscriptionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        subscription: WebhookSubscriptionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateWebhookSubscriptionResponse]:
        """
        Creates a webhook subscription.

        Parameters
        ----------
        subscription : WebhookSubscriptionParams
            The [Subscription](entity:WebhookSubscription) to create.

        idempotency_key : typing.Optional[str]
            A unique string that identifies the [CreateWebhookSubscription](api-endpoint:WebhookSubscriptions-CreateWebhookSubscription) request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateWebhookSubscriptionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/webhooks/subscriptions",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "subscription": convert_and_respect_annotation_metadata(
                    object_=subscription, annotation=WebhookSubscriptionParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateWebhookSubscriptionResponse,
                    construct_type(
                        type_=CreateWebhookSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, subscription_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetWebhookSubscriptionResponse]:
        """
        Retrieves a webhook subscription identified by its ID.

        Parameters
        ----------
        subscription_id : str
            [REQUIRED] The ID of the [Subscription](entity:WebhookSubscription) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetWebhookSubscriptionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/webhooks/subscriptions/{jsonable_encoder(subscription_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetWebhookSubscriptionResponse,
                    construct_type(
                        type_=GetWebhookSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        subscription_id: str,
        *,
        subscription: typing.Optional[WebhookSubscriptionParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateWebhookSubscriptionResponse]:
        """
        Updates a webhook subscription.

        Parameters
        ----------
        subscription_id : str
            [REQUIRED] The ID of the [Subscription](entity:WebhookSubscription) to update.

        subscription : typing.Optional[WebhookSubscriptionParams]
            The [Subscription](entity:WebhookSubscription) to update.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateWebhookSubscriptionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/webhooks/subscriptions/{jsonable_encoder(subscription_id)}",
            method="PUT",
            json={
                "subscription": convert_and_respect_annotation_metadata(
                    object_=subscription, annotation=WebhookSubscriptionParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateWebhookSubscriptionResponse,
                    construct_type(
                        type_=UpdateWebhookSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self, subscription_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteWebhookSubscriptionResponse]:
        """
        Deletes a webhook subscription.

        Parameters
        ----------
        subscription_id : str
            [REQUIRED] The ID of the [Subscription](entity:WebhookSubscription) to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteWebhookSubscriptionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/webhooks/subscriptions/{jsonable_encoder(subscription_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteWebhookSubscriptionResponse,
                    construct_type(
                        type_=DeleteWebhookSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_signature_key(
        self,
        subscription_id: str,
        *,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateWebhookSubscriptionSignatureKeyResponse]:
        """
        Updates a webhook subscription by replacing the existing signature key with a new one.

        Parameters
        ----------
        subscription_id : str
            [REQUIRED] The ID of the [Subscription](entity:WebhookSubscription) to update.

        idempotency_key : typing.Optional[str]
            A unique string that identifies the [UpdateWebhookSubscriptionSignatureKey](api-endpoint:WebhookSubscriptions-UpdateWebhookSubscriptionSignatureKey) request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateWebhookSubscriptionSignatureKeyResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/webhooks/subscriptions/{jsonable_encoder(subscription_id)}/signature-key",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateWebhookSubscriptionSignatureKeyResponse,
                    construct_type(
                        type_=UpdateWebhookSubscriptionSignatureKeyResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def test(
        self,
        subscription_id: str,
        *,
        event_type: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[TestWebhookSubscriptionResponse]:
        """
        Tests a webhook subscription by sending a test event to the notification URL.

        Parameters
        ----------
        subscription_id : str
            [REQUIRED] The ID of the [Subscription](entity:WebhookSubscription) to test.

        event_type : typing.Optional[str]
            The event type that will be used to test the [Subscription](entity:WebhookSubscription). The event type must be
            contained in the list of event types in the [Subscription](entity:WebhookSubscription).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TestWebhookSubscriptionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/webhooks/subscriptions/{jsonable_encoder(subscription_id)}/test",
            method="POST",
            json={
                "event_type": event_type,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TestWebhookSubscriptionResponse,
                    construct_type(
                        type_=TestWebhookSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawSubscriptionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        subscription: WebhookSubscriptionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateWebhookSubscriptionResponse]:
        """
        Creates a webhook subscription.

        Parameters
        ----------
        subscription : WebhookSubscriptionParams
            The [Subscription](entity:WebhookSubscription) to create.

        idempotency_key : typing.Optional[str]
            A unique string that identifies the [CreateWebhookSubscription](api-endpoint:WebhookSubscriptions-CreateWebhookSubscription) request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateWebhookSubscriptionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/webhooks/subscriptions",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "subscription": convert_and_respect_annotation_metadata(
                    object_=subscription, annotation=WebhookSubscriptionParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateWebhookSubscriptionResponse,
                    construct_type(
                        type_=CreateWebhookSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, subscription_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetWebhookSubscriptionResponse]:
        """
        Retrieves a webhook subscription identified by its ID.

        Parameters
        ----------
        subscription_id : str
            [REQUIRED] The ID of the [Subscription](entity:WebhookSubscription) to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetWebhookSubscriptionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/webhooks/subscriptions/{jsonable_encoder(subscription_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetWebhookSubscriptionResponse,
                    construct_type(
                        type_=GetWebhookSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        subscription_id: str,
        *,
        subscription: typing.Optional[WebhookSubscriptionParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateWebhookSubscriptionResponse]:
        """
        Updates a webhook subscription.

        Parameters
        ----------
        subscription_id : str
            [REQUIRED] The ID of the [Subscription](entity:WebhookSubscription) to update.

        subscription : typing.Optional[WebhookSubscriptionParams]
            The [Subscription](entity:WebhookSubscription) to update.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateWebhookSubscriptionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/webhooks/subscriptions/{jsonable_encoder(subscription_id)}",
            method="PUT",
            json={
                "subscription": convert_and_respect_annotation_metadata(
                    object_=subscription, annotation=WebhookSubscriptionParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateWebhookSubscriptionResponse,
                    construct_type(
                        type_=UpdateWebhookSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, subscription_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteWebhookSubscriptionResponse]:
        """
        Deletes a webhook subscription.

        Parameters
        ----------
        subscription_id : str
            [REQUIRED] The ID of the [Subscription](entity:WebhookSubscription) to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteWebhookSubscriptionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/webhooks/subscriptions/{jsonable_encoder(subscription_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteWebhookSubscriptionResponse,
                    construct_type(
                        type_=DeleteWebhookSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_signature_key(
        self,
        subscription_id: str,
        *,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateWebhookSubscriptionSignatureKeyResponse]:
        """
        Updates a webhook subscription by replacing the existing signature key with a new one.

        Parameters
        ----------
        subscription_id : str
            [REQUIRED] The ID of the [Subscription](entity:WebhookSubscription) to update.

        idempotency_key : typing.Optional[str]
            A unique string that identifies the [UpdateWebhookSubscriptionSignatureKey](api-endpoint:WebhookSubscriptions-UpdateWebhookSubscriptionSignatureKey) request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateWebhookSubscriptionSignatureKeyResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/webhooks/subscriptions/{jsonable_encoder(subscription_id)}/signature-key",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateWebhookSubscriptionSignatureKeyResponse,
                    construct_type(
                        type_=UpdateWebhookSubscriptionSignatureKeyResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def test(
        self,
        subscription_id: str,
        *,
        event_type: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[TestWebhookSubscriptionResponse]:
        """
        Tests a webhook subscription by sending a test event to the notification URL.

        Parameters
        ----------
        subscription_id : str
            [REQUIRED] The ID of the [Subscription](entity:WebhookSubscription) to test.

        event_type : typing.Optional[str]
            The event type that will be used to test the [Subscription](entity:WebhookSubscription). The event type must be
            contained in the list of event types in the [Subscription](entity:WebhookSubscription).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TestWebhookSubscriptionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/webhooks/subscriptions/{jsonable_encoder(subscription_id)}/test",
            method="POST",
            json={
                "event_type": event_type,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TestWebhookSubscriptionResponse,
                    construct_type(
                        type_=TestWebhookSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .invoice import InvoiceParams
import typing
from .error import ErrorParams


class PublishInvoiceResponseParams(typing_extensions.TypedDict):
    """
    Describes a `PublishInvoice` response.
    """

    invoice: typing_extensions.NotRequired[InvoiceParams]
    """
    The published invoice.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information about errors encountered during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams


class DeletePaymentLinkResponseParams(typing_extensions.TypedDict):
    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    id: typing_extensions.NotRequired[str]
    """
    The ID of the link that is deleted.
    """

    cancelled_order_id: typing_extensions.NotRequired[str]
    """
    The ID of the order that is canceled. When a payment link is deleted, Square updates the
    the `state` (of the order that the checkout link created) to CANCELED.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .address import AddressParams
from .vendor_contact import VendorContactParams
from ..types.vendor_status import VendorStatus


class VendorParams(typing_extensions.TypedDict):
    """
    Represents a supplier to a seller.
    """

    id: typing_extensions.NotRequired[str]
    """
    A unique Square-generated ID for the [Vendor](entity:Vendor).
    This field is required when attempting to update a [Vendor](entity:Vendor).
    """

    created_at: typing_extensions.NotRequired[str]
    """
    An RFC 3339-formatted timestamp that indicates when the
    [Vendor](entity:Vendor) was created.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    An RFC 3339-formatted timestamp that indicates when the
    [Vendor](entity:Vendor) was last updated.
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of the [Vendor](entity:Vendor).
    This field is required when attempting to create or update a [Vendor](entity:Vendor).
    """

    address: typing_extensions.NotRequired[AddressParams]
    """
    The address of the [Vendor](entity:Vendor).
    """

    contacts: typing_extensions.NotRequired[typing.Optional[typing.Sequence[VendorContactParams]]]
    """
    The contacts of the [Vendor](entity:Vendor).
    """

    account_number: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The account number of the [Vendor](entity:Vendor).
    """

    note: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A note detailing information about the [Vendor](entity:Vendor).
    """

    version: typing_extensions.NotRequired[int]
    """
    The version of the [Vendor](entity:Vendor).
    """

    status: typing_extensions.NotRequired[VendorStatus]
    """
    The status of the [Vendor](entity:Vendor).
    See [Status](#type-status) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.loyalty_program_accrual_rule_type import LoyaltyProgramAccrualRuleType
import typing_extensions
import typing
from .loyalty_program_accrual_rule_visit_data import LoyaltyProgramAccrualRuleVisitDataParams
from .loyalty_program_accrual_rule_spend_data import LoyaltyProgramAccrualRuleSpendDataParams
from .loyalty_program_accrual_rule_item_variation_data import LoyaltyProgramAccrualRuleItemVariationDataParams
from .loyalty_program_accrual_rule_category_data import LoyaltyProgramAccrualRuleCategoryDataParams


class LoyaltyProgramAccrualRuleParams(typing_extensions.TypedDict):
    """
    Represents an accrual rule, which defines how buyers can earn points from the base [loyalty program](entity:LoyaltyProgram).
    """

    accrual_type: LoyaltyProgramAccrualRuleType
    """
    The type of the accrual rule that defines how buyers can earn points.
    See [LoyaltyProgramAccrualRuleType](#type-loyaltyprogramaccrualruletype) for possible values
    """

    points: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The number of points that 
    buyers earn based on the `accrual_type`.
    """

    visit_data: typing_extensions.NotRequired[LoyaltyProgramAccrualRuleVisitDataParams]
    """
    Additional data for rules with the `VISIT` accrual type.
    """

    spend_data: typing_extensions.NotRequired[LoyaltyProgramAccrualRuleSpendDataParams]
    """
    Additional data for rules with the `SPEND` accrual type.
    """

    item_variation_data: typing_extensions.NotRequired[LoyaltyProgramAccrualRuleItemVariationDataParams]
    """
    Additional data for rules with the `ITEM_VARIATION` accrual type.
    """

    category_data: typing_extensions.NotRequired[LoyaltyProgramAccrualRuleCategoryDataParams]
    """
    Additional data for rules with the `CATEGORY` accrual type.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.search_orders_sort_field import SearchOrdersSortField
import typing_extensions
from ..types.sort_order import SortOrder


class SearchOrdersSortParams(typing_extensions.TypedDict):
    """
    Sorting criteria for a `SearchOrders` request. Results can only be sorted
    by a timestamp field.
    """

    sort_field: SearchOrdersSortField
    """
    The field to sort by.
    
    __Important:__ When using a [DateTimeFilter](entity:SearchOrdersFilter),
    `sort_field` must match the timestamp field that the `DateTimeFilter` uses to
    filter. For example, if you set your `sort_field` to `CLOSED_AT` and you use a
    `DateTimeFilter`, your `DateTimeFilter` must filter for orders by their `CLOSED_AT` date.
    If this field does not match the timestamp field in `DateTimeFilter`,
    `SearchOrders` returns an error.
    
    Default: `CREATED_AT`.
    See [SearchOrdersSortField](#type-searchorderssortfield) for possible values
    """

    sort_order: typing_extensions.NotRequired[SortOrder]
    """
    The chronological order in which results are returned. Defaults to `DESC`.
    See [SortOrder](#type-sortorder) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.sort_order import SortOrder


class TerminalActionQuerySortParams(typing_extensions.TypedDict):
    sort_order: typing_extensions.NotRequired[SortOrder]
    """
    The order in which results are listed.
    - `ASC` - Oldest to newest.
    - `DESC` - Newest to oldest (default).
    See [SortOrder](#type-sortorder) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawApplePayClient
from ..core.request_options import RequestOptions
from ..types.register_domain_response import RegisterDomainResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawApplePayClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ApplePayClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawApplePayClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawApplePayClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawApplePayClient
        """
        return self._raw_client

    def register_domain(
        self, *, domain_name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> RegisterDomainResponse:
        """
        Activates a domain for use with Apple Pay on the Web and Square. A validation
        is performed on this domain by Apple to ensure that it is properly set up as
        an Apple Pay enabled domain.

        This endpoint provides an easy way for platform developers to bulk activate
        Apple Pay on the Web with Square for merchants using their platform.

        Note: You will need to host a valid domain verification file on your domain to support Apple Pay.  The
        current version of this file is always available at https://app.squareup.com/digital-wallets/apple-pay/apple-developer-merchantid-domain-association,
        and should be hosted at `.well_known/apple-developer-merchantid-domain-association` on your
        domain.  This file is subject to change; we strongly recommend checking for updates regularly and avoiding
        long-lived caches that might not keep in sync with the correct file version.

        To learn more about the Web Payments SDK and how to add Apple Pay, see [Take an Apple Pay Payment](https://developer.squareup.com/docs/web-payments/apple-pay).

        Parameters
        ----------
        domain_name : str
            A domain name as described in RFC-1034 that will be registered with ApplePay.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RegisterDomainResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.apple_pay.register_domain(
            domain_name="example.com",
        )
        """
        response = self._raw_client.register_domain(domain_name=domain_name, request_options=request_options)
        return response.data


class AsyncApplePayClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawApplePayClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawApplePayClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawApplePayClient
        """
        return self._raw_client

    async def register_domain(
        self, *, domain_name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> RegisterDomainResponse:
        """
        Activates a domain for use with Apple Pay on the Web and Square. A validation
        is performed on this domain by Apple to ensure that it is properly set up as
        an Apple Pay enabled domain.

        This endpoint provides an easy way for platform developers to bulk activate
        Apple Pay on the Web with Square for merchants using their platform.

        Note: You will need to host a valid domain verification file on your domain to support Apple Pay.  The
        current version of this file is always available at https://app.squareup.com/digital-wallets/apple-pay/apple-developer-merchantid-domain-association,
        and should be hosted at `.well_known/apple-developer-merchantid-domain-association` on your
        domain.  This file is subject to change; we strongly recommend checking for updates regularly and avoiding
        long-lived caches that might not keep in sync with the correct file version.

        To learn more about the Web Payments SDK and how to add Apple Pay, see [Take an Apple Pay Payment](https://developer.squareup.com/docs/web-payments/apple-pay).

        Parameters
        ----------
        domain_name : str
            A domain name as described in RFC-1034 that will be registered with ApplePay.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RegisterDomainResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.apple_pay.register_domain(
                domain_name="example.com",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.register_domain(domain_name=domain_name, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .custom_attribute_definition import CustomAttributeDefinitionParams
import typing
from .error import ErrorParams


class CreateLocationCustomAttributeDefinitionResponseParams(typing_extensions.TypedDict):
    """
    Represents a [CreateLocationCustomAttributeDefinition](api-endpoint:LocationCustomAttributes-CreateLocationCustomAttributeDefinition) response.
    Either `custom_attribute_definition` or `errors` is present in the response.
    """

    custom_attribute_definition: typing_extensions.NotRequired[CustomAttributeDefinitionParams]
    """
    The new custom attribute definition.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.shift_sort_field import ShiftSortField
from ..types.sort_order import SortOrder


class ShiftSortParams(typing_extensions.TypedDict):
    """
    Sets the sort order of search results.

    Deprecated at Square API version 2025-05-21. See the [migration notes](https://developer.squareup.com/docs/labor-api/what-it-does#migration-notes).
    """

    field: typing_extensions.NotRequired[ShiftSortField]
    """
    The field to sort on.
    See [ShiftSortField](#type-shiftsortfield) for possible values
    """

    order: typing_extensions.NotRequired[SortOrder]
    """
    The order in which results are returned. Defaults to DESC.
    See [SortOrder](#type-sortorder) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
from .component_component_type import ComponentComponentType
import pydantic
import typing
from .device_component_details_application_details import DeviceComponentDetailsApplicationDetails
from .device_component_details_card_reader_details import DeviceComponentDetailsCardReaderDetails
from .device_component_details_battery_details import DeviceComponentDetailsBatteryDetails
from .device_component_details_wi_fi_details import DeviceComponentDetailsWiFiDetails
from .device_component_details_ethernet_details import DeviceComponentDetailsEthernetDetails
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Component(UncheckedBaseModel):
    """
    The wrapper object for the component entries of a given component type.
    """

    type: ComponentComponentType = pydantic.Field()
    """
    The type of this component. Each component type has expected properties expressed
    in a structured format within its corresponding `*_details` field.
    See [ComponentType](#type-componenttype) for possible values
    """

    application_details: typing.Optional[DeviceComponentDetailsApplicationDetails] = pydantic.Field(default=None)
    """
    Structured data for an `Application`, set for Components of type `APPLICATION`.
    """

    card_reader_details: typing.Optional[DeviceComponentDetailsCardReaderDetails] = pydantic.Field(default=None)
    """
    Structured data for a `CardReader`, set for Components of type `CARD_READER`.
    """

    battery_details: typing.Optional[DeviceComponentDetailsBatteryDetails] = pydantic.Field(default=None)
    """
    Structured data for a `Battery`, set for Components of type `BATTERY`.
    """

    wifi_details: typing.Optional[DeviceComponentDetailsWiFiDetails] = pydantic.Field(default=None)
    """
    Structured data for a `WiFi` interface, set for Components of type `WIFI`.
    """

    ethernet_details: typing.Optional[DeviceComponentDetailsEthernetDetails] = pydantic.Field(default=None)
    """
    Structured data for an `Ethernet` interface, set for Components of type `ETHERNET`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.loyalty_reward_status import LoyaltyRewardStatus
import typing


class LoyaltyRewardParams(typing_extensions.TypedDict):
    """
    Represents a contract to redeem loyalty points for a [reward tier](entity:LoyaltyProgramRewardTier) discount. Loyalty rewards can be in an ISSUED, REDEEMED, or DELETED state.
    For more information, see [Manage loyalty rewards](https://developer.squareup.com/docs/loyalty-api/loyalty-rewards).
    """

    id: typing_extensions.NotRequired[str]
    """
    The Square-assigned ID of the loyalty reward.
    """

    status: typing_extensions.NotRequired[LoyaltyRewardStatus]
    """
    The status of a loyalty reward.
    See [LoyaltyRewardStatus](#type-loyaltyrewardstatus) for possible values
    """

    loyalty_account_id: str
    """
    The Square-assigned ID of the [loyalty account](entity:LoyaltyAccount) to which the reward belongs.
    """

    reward_tier_id: str
    """
    The Square-assigned ID of the [reward tier](entity:LoyaltyProgramRewardTier) used to create the reward.
    """

    points: typing_extensions.NotRequired[int]
    """
    The number of loyalty points used for the reward.
    """

    order_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The Square-assigned ID of the [order](entity:Order) to which the reward is attached.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the reward was created, in RFC 3339 format.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the reward was last updated, in RFC 3339 format.
    """

    redeemed_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the reward was redeemed, in RFC 3339 format.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions


class CollectedDataParams(typing_extensions.TypedDict):
    input_text: typing_extensions.NotRequired[str]
    """
    The buyer's input text.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawCustomAttributesClient
from ...requests.bulk_delete_location_custom_attributes_request_location_custom_attribute_delete_request import (
    BulkDeleteLocationCustomAttributesRequestLocationCustomAttributeDeleteRequestParams,
)
from ...core.request_options import RequestOptions
from ...types.bulk_delete_location_custom_attributes_response import BulkDeleteLocationCustomAttributesResponse
from ...requests.bulk_upsert_location_custom_attributes_request_location_custom_attribute_upsert_request import (
    BulkUpsertLocationCustomAttributesRequestLocationCustomAttributeUpsertRequestParams,
)
from ...types.bulk_upsert_location_custom_attributes_response import BulkUpsertLocationCustomAttributesResponse
from ...types.visibility_filter import VisibilityFilter
from ...core.pagination import SyncPager
from ...types.custom_attribute import CustomAttribute
from ...core.jsonable_encoder import jsonable_encoder
from ...types.list_location_custom_attributes_response import ListLocationCustomAttributesResponse
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...types.retrieve_location_custom_attribute_response import RetrieveLocationCustomAttributeResponse
from ...requests.custom_attribute import CustomAttributeParams
from ...types.upsert_location_custom_attribute_response import UpsertLocationCustomAttributeResponse
from ...types.delete_location_custom_attribute_response import DeleteLocationCustomAttributeResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawCustomAttributesClient
from ...core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CustomAttributesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCustomAttributesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCustomAttributesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCustomAttributesClient
        """
        return self._raw_client

    def batch_delete(
        self,
        *,
        values: typing.Dict[str, BulkDeleteLocationCustomAttributesRequestLocationCustomAttributeDeleteRequestParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkDeleteLocationCustomAttributesResponse:
        """
        Deletes [custom attributes](entity:CustomAttribute) for locations as a bulk operation.
        To delete a custom attribute owned by another application, the `visibility` setting must be
        `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        values : typing.Dict[str, BulkDeleteLocationCustomAttributesRequestLocationCustomAttributeDeleteRequestParams]
            The data used to update the `CustomAttribute` objects.
            The keys must be unique and are used to map to the corresponding response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkDeleteLocationCustomAttributesResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.locations.custom_attributes.batch_delete(
            values={
                "id1": {"key": "bestseller"},
                "id2": {"key": "bestseller"},
                "id3": {"key": "phone-number"},
            },
        )
        """
        response = self._raw_client.batch_delete(values=values, request_options=request_options)
        return response.data

    def batch_upsert(
        self,
        *,
        values: typing.Dict[str, BulkUpsertLocationCustomAttributesRequestLocationCustomAttributeUpsertRequestParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkUpsertLocationCustomAttributesResponse:
        """
        Creates or updates [custom attributes](entity:CustomAttribute) for locations as a bulk operation.
        Use this endpoint to set the value of one or more custom attributes for one or more locations.
        A custom attribute is based on a custom attribute definition in a Square seller account, which is
        created using the [CreateLocationCustomAttributeDefinition](api-endpoint:LocationCustomAttributes-CreateLocationCustomAttributeDefinition) endpoint.
        This `BulkUpsertLocationCustomAttributes` endpoint accepts a map of 1 to 25 individual upsert
        requests and returns a map of individual upsert responses. Each upsert request has a unique ID
        and provides a location ID and custom attribute. Each upsert response is returned with the ID
        of the corresponding request.
        To create or update a custom attribute owned by another application, the `visibility` setting
        must be `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        values : typing.Dict[str, BulkUpsertLocationCustomAttributesRequestLocationCustomAttributeUpsertRequestParams]
            A map containing 1 to 25 individual upsert requests. For each request, provide an
            arbitrary ID that is unique for this `BulkUpsertLocationCustomAttributes` request and the
            information needed to create or update a custom attribute.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkUpsertLocationCustomAttributesResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.locations.custom_attributes.batch_upsert(
            values={
                "id1": {
                    "location_id": "L0TBCBTB7P8RQ",
                    "custom_attribute": {"key": "bestseller", "value": "hot cocoa"},
                },
                "id2": {
                    "location_id": "L9XMD04V3STJX",
                    "custom_attribute": {
                        "key": "bestseller",
                        "value": "berry smoothie",
                    },
                },
                "id3": {
                    "location_id": "L0TBCBTB7P8RQ",
                    "custom_attribute": {
                        "key": "phone-number",
                        "value": "+12223334444",
                    },
                },
            },
        )
        """
        response = self._raw_client.batch_upsert(values=values, request_options=request_options)
        return response.data

    def list(
        self,
        location_id: str,
        *,
        visibility_filter: typing.Optional[VisibilityFilter] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        with_definitions: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[CustomAttribute]:
        """
        Lists the [custom attributes](entity:CustomAttribute) associated with a location.
        You can use the `with_definitions` query parameter to also retrieve custom attribute definitions
        in the same call.
        When all response pages are retrieved, the results include all custom attributes that are
        visible to the requesting application, including those that are owned by other applications
        and set to `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        location_id : str
            The ID of the target [location](entity:Location).

        visibility_filter : typing.Optional[VisibilityFilter]
            Filters the `CustomAttributeDefinition` results by their `visibility` values.

        limit : typing.Optional[int]
            The maximum number of results to return in a single paged response. This limit is advisory.
            The response might contain more or fewer results. The minimum value is 1 and the maximum value is 100.
            The default value is 20. For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        cursor : typing.Optional[str]
            The cursor returned in the paged response from the previous call to this endpoint.
            Provide this cursor to retrieve the next page of results for your original request. For more
            information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        with_definitions : typing.Optional[bool]
            Indicates whether to return the [custom attribute definition](entity:CustomAttributeDefinition) in the `definition` field of each
            custom attribute. Set this parameter to `true` to get the name and description of each custom
            attribute, information about the data type, or other definition details. The default value is `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[CustomAttribute]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.locations.custom_attributes.list(
            location_id="location_id",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            f"v2/locations/{jsonable_encoder(location_id)}/custom-attributes",
            method="GET",
            params={
                "visibility_filter": visibility_filter,
                "limit": limit,
                "cursor": cursor,
                "with_definitions": with_definitions,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListLocationCustomAttributesResponse,
                    construct_type(
                        type_=ListLocationCustomAttributesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    location_id,
                    visibility_filter=visibility_filter,
                    limit=limit,
                    cursor=_parsed_next,
                    with_definitions=with_definitions,
                    request_options=request_options,
                )
                _items = _parsed_response.custom_attributes
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self,
        location_id: str,
        key: str,
        *,
        with_definition: typing.Optional[bool] = None,
        version: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RetrieveLocationCustomAttributeResponse:
        """
        Retrieves a [custom attribute](entity:CustomAttribute) associated with a location.
        You can use the `with_definition` query parameter to also retrieve the custom attribute definition
        in the same call.
        To retrieve a custom attribute owned by another application, the `visibility` setting must be
        `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        location_id : str
            The ID of the target [location](entity:Location).

        key : str
            The key of the custom attribute to retrieve. This key must match the `key` of a custom
            attribute definition in the Square seller account. If the requesting application is not the
            definition owner, you must use the qualified key.

        with_definition : typing.Optional[bool]
            Indicates whether to return the [custom attribute definition](entity:CustomAttributeDefinition) in the `definition` field of
            the custom attribute. Set this parameter to `true` to get the name and description of the custom
            attribute, information about the data type, or other definition details. The default value is `false`.

        version : typing.Optional[int]
            The current version of the custom attribute, which is used for strongly consistent reads to
            guarantee that you receive the most up-to-date data. When included in the request, Square
            returns the specified version or a higher version if one exists. If the specified version is
            higher than the current version, Square returns a `BAD_REQUEST` error.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveLocationCustomAttributeResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.locations.custom_attributes.get(
            location_id="location_id",
            key="key",
        )
        """
        response = self._raw_client.get(
            location_id, key, with_definition=with_definition, version=version, request_options=request_options
        )
        return response.data

    def upsert(
        self,
        location_id: str,
        key: str,
        *,
        custom_attribute: CustomAttributeParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpsertLocationCustomAttributeResponse:
        """
        Creates or updates a [custom attribute](entity:CustomAttribute) for a location.
        Use this endpoint to set the value of a custom attribute for a specified location.
        A custom attribute is based on a custom attribute definition in a Square seller account, which
        is created using the [CreateLocationCustomAttributeDefinition](api-endpoint:LocationCustomAttributes-CreateLocationCustomAttributeDefinition) endpoint.
        To create or update a custom attribute owned by another application, the `visibility` setting
        must be `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        location_id : str
            The ID of the target [location](entity:Location).

        key : str
            The key of the custom attribute to create or update. This key must match the `key` of a
            custom attribute definition in the Square seller account. If the requesting application is not
            the definition owner, you must use the qualified key.

        custom_attribute : CustomAttributeParams
            The custom attribute to create or update, with the following fields:
            - `value`. This value must conform to the `schema` specified by the definition.
            For more information, see [Supported data types](https://developer.squareup.com/docs/devtools/customattributes/overview#supported-data-types).
            - `version`. To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control for an update operation, include the current version of the custom attribute.
            If this is not important for your application, version can be set to -1.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpsertLocationCustomAttributeResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.locations.custom_attributes.upsert(
            location_id="location_id",
            key="key",
            custom_attribute={"value": "hot cocoa"},
        )
        """
        response = self._raw_client.upsert(
            location_id,
            key,
            custom_attribute=custom_attribute,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    def delete(
        self, location_id: str, key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteLocationCustomAttributeResponse:
        """
        Deletes a [custom attribute](entity:CustomAttribute) associated with a location.
        To delete a custom attribute owned by another application, the `visibility` setting must be
        `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        location_id : str
            The ID of the target [location](entity:Location).

        key : str
            The key of the custom attribute to delete. This key must match the `key` of a custom
            attribute definition in the Square seller account. If the requesting application is not the
            definition owner, you must use the qualified key.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteLocationCustomAttributeResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.locations.custom_attributes.delete(
            location_id="location_id",
            key="key",
        )
        """
        response = self._raw_client.delete(location_id, key, request_options=request_options)
        return response.data


class AsyncCustomAttributesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCustomAttributesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCustomAttributesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCustomAttributesClient
        """
        return self._raw_client

    async def batch_delete(
        self,
        *,
        values: typing.Dict[str, BulkDeleteLocationCustomAttributesRequestLocationCustomAttributeDeleteRequestParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkDeleteLocationCustomAttributesResponse:
        """
        Deletes [custom attributes](entity:CustomAttribute) for locations as a bulk operation.
        To delete a custom attribute owned by another application, the `visibility` setting must be
        `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        values : typing.Dict[str, BulkDeleteLocationCustomAttributesRequestLocationCustomAttributeDeleteRequestParams]
            The data used to update the `CustomAttribute` objects.
            The keys must be unique and are used to map to the corresponding response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkDeleteLocationCustomAttributesResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.locations.custom_attributes.batch_delete(
                values={
                    "id1": {"key": "bestseller"},
                    "id2": {"key": "bestseller"},
                    "id3": {"key": "phone-number"},
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.batch_delete(values=values, request_options=request_options)
        return response.data

    async def batch_upsert(
        self,
        *,
        values: typing.Dict[str, BulkUpsertLocationCustomAttributesRequestLocationCustomAttributeUpsertRequestParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkUpsertLocationCustomAttributesResponse:
        """
        Creates or updates [custom attributes](entity:CustomAttribute) for locations as a bulk operation.
        Use this endpoint to set the value of one or more custom attributes for one or more locations.
        A custom attribute is based on a custom attribute definition in a Square seller account, which is
        created using the [CreateLocationCustomAttributeDefinition](api-endpoint:LocationCustomAttributes-CreateLocationCustomAttributeDefinition) endpoint.
        This `BulkUpsertLocationCustomAttributes` endpoint accepts a map of 1 to 25 individual upsert
        requests and returns a map of individual upsert responses. Each upsert request has a unique ID
        and provides a location ID and custom attribute. Each upsert response is returned with the ID
        of the corresponding request.
        To create or update a custom attribute owned by another application, the `visibility` setting
        must be `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        values : typing.Dict[str, BulkUpsertLocationCustomAttributesRequestLocationCustomAttributeUpsertRequestParams]
            A map containing 1 to 25 individual upsert requests. For each request, provide an
            arbitrary ID that is unique for this `BulkUpsertLocationCustomAttributes` request and the
            information needed to create or update a custom attribute.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkUpsertLocationCustomAttributesResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.locations.custom_attributes.batch_upsert(
                values={
                    "id1": {
                        "location_id": "L0TBCBTB7P8RQ",
                        "custom_attribute": {"key": "bestseller", "value": "hot cocoa"},
                    },
                    "id2": {
                        "location_id": "L9XMD04V3STJX",
                        "custom_attribute": {
                            "key": "bestseller",
                            "value": "berry smoothie",
                        },
                    },
                    "id3": {
                        "location_id": "L0TBCBTB7P8RQ",
                        "custom_attribute": {
                            "key": "phone-number",
                            "value": "+12223334444",
                        },
                    },
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.batch_upsert(values=values, request_options=request_options)
        return response.data

    async def list(
        self,
        location_id: str,
        *,
        visibility_filter: typing.Optional[VisibilityFilter] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        with_definitions: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[CustomAttribute]:
        """
        Lists the [custom attributes](entity:CustomAttribute) associated with a location.
        You can use the `with_definitions` query parameter to also retrieve custom attribute definitions
        in the same call.
        When all response pages are retrieved, the results include all custom attributes that are
        visible to the requesting application, including those that are owned by other applications
        and set to `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        location_id : str
            The ID of the target [location](entity:Location).

        visibility_filter : typing.Optional[VisibilityFilter]
            Filters the `CustomAttributeDefinition` results by their `visibility` values.

        limit : typing.Optional[int]
            The maximum number of results to return in a single paged response. This limit is advisory.
            The response might contain more or fewer results. The minimum value is 1 and the maximum value is 100.
            The default value is 20. For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        cursor : typing.Optional[str]
            The cursor returned in the paged response from the previous call to this endpoint.
            Provide this cursor to retrieve the next page of results for your original request. For more
            information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        with_definitions : typing.Optional[bool]
            Indicates whether to return the [custom attribute definition](entity:CustomAttributeDefinition) in the `definition` field of each
            custom attribute. Set this parameter to `true` to get the name and description of each custom
            attribute, information about the data type, or other definition details. The default value is `false`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[CustomAttribute]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.locations.custom_attributes.list(
                location_id="location_id",
            )
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            f"v2/locations/{jsonable_encoder(location_id)}/custom-attributes",
            method="GET",
            params={
                "visibility_filter": visibility_filter,
                "limit": limit,
                "cursor": cursor,
                "with_definitions": with_definitions,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListLocationCustomAttributesResponse,
                    construct_type(
                        type_=ListLocationCustomAttributesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    location_id,
                    visibility_filter=visibility_filter,
                    limit=limit,
                    cursor=_parsed_next,
                    with_definitions=with_definitions,
                    request_options=request_options,
                )
                _items = _parsed_response.custom_attributes
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self,
        location_id: str,
        key: str,
        *,
        with_definition: typing.Optional[bool] = None,
        version: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RetrieveLocationCustomAttributeResponse:
        """
        Retrieves a [custom attribute](entity:CustomAttribute) associated with a location.
        You can use the `with_definition` query parameter to also retrieve the custom attribute definition
        in the same call.
        To retrieve a custom attribute owned by another application, the `visibility` setting must be
        `VISIBILITY_READ_ONLY` or `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        location_id : str
            The ID of the target [location](entity:Location).

        key : str
            The key of the custom attribute to retrieve. This key must match the `key` of a custom
            attribute definition in the Square seller account. If the requesting application is not the
            definition owner, you must use the qualified key.

        with_definition : typing.Optional[bool]
            Indicates whether to return the [custom attribute definition](entity:CustomAttributeDefinition) in the `definition` field of
            the custom attribute. Set this parameter to `true` to get the name and description of the custom
            attribute, information about the data type, or other definition details. The default value is `false`.

        version : typing.Optional[int]
            The current version of the custom attribute, which is used for strongly consistent reads to
            guarantee that you receive the most up-to-date data. When included in the request, Square
            returns the specified version or a higher version if one exists. If the specified version is
            higher than the current version, Square returns a `BAD_REQUEST` error.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RetrieveLocationCustomAttributeResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.locations.custom_attributes.get(
                location_id="location_id",
                key="key",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(
            location_id, key, with_definition=with_definition, version=version, request_options=request_options
        )
        return response.data

    async def upsert(
        self,
        location_id: str,
        key: str,
        *,
        custom_attribute: CustomAttributeParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpsertLocationCustomAttributeResponse:
        """
        Creates or updates a [custom attribute](entity:CustomAttribute) for a location.
        Use this endpoint to set the value of a custom attribute for a specified location.
        A custom attribute is based on a custom attribute definition in a Square seller account, which
        is created using the [CreateLocationCustomAttributeDefinition](api-endpoint:LocationCustomAttributes-CreateLocationCustomAttributeDefinition) endpoint.
        To create or update a custom attribute owned by another application, the `visibility` setting
        must be `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        location_id : str
            The ID of the target [location](entity:Location).

        key : str
            The key of the custom attribute to create or update. This key must match the `key` of a
            custom attribute definition in the Square seller account. If the requesting application is not
            the definition owner, you must use the qualified key.

        custom_attribute : CustomAttributeParams
            The custom attribute to create or update, with the following fields:
            - `value`. This value must conform to the `schema` specified by the definition.
            For more information, see [Supported data types](https://developer.squareup.com/docs/devtools/customattributes/overview#supported-data-types).
            - `version`. To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control for an update operation, include the current version of the custom attribute.
            If this is not important for your application, version can be set to -1.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpsertLocationCustomAttributeResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.locations.custom_attributes.upsert(
                location_id="location_id",
                key="key",
                custom_attribute={"value": "hot cocoa"},
            )


        asyncio.run(main())
        """
        response = await self._raw_client.upsert(
            location_id,
            key,
            custom_attribute=custom_attribute,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    async def delete(
        self, location_id: str, key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteLocationCustomAttributeResponse:
        """
        Deletes a [custom attribute](entity:CustomAttribute) associated with a location.
        To delete a custom attribute owned by another application, the `visibility` setting must be
        `VISIBILITY_READ_WRITE_VALUES`.

        Parameters
        ----------
        location_id : str
            The ID of the target [location](entity:Location).

        key : str
            The key of the custom attribute to delete. This key must match the `key` of a custom
            attribute definition in the Square seller account. If the requesting application is not the
            definition owner, you must use the qualified key.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteLocationCustomAttributeResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.locations.custom_attributes.delete(
                location_id="location_id",
                key="key",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.delete(location_id, key, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing

GiftCardActivityClearBalanceReason = typing.Union[
    typing.Literal["SUSPICIOUS_ACTIVITY", "REUSE_GIFTCARD", "UNKNOWN_REASON"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .order_return_line_item import OrderReturnLineItem
from .order_return_service_charge import OrderReturnServiceCharge
from .order_return_tax import OrderReturnTax
from .order_return_discount import OrderReturnDiscount
from .order_return_tip import OrderReturnTip
from .order_rounding_adjustment import OrderRoundingAdjustment
from .order_money_amounts import OrderMoneyAmounts
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class OrderReturn(UncheckedBaseModel):
    """
    The set of line items, service charges, taxes, discounts, tips, and other items being returned in an order.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique ID that identifies the return only within this order.
    """

    source_order_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    An order that contains the original sale of these return line items. This is unset
    for unlinked returns.
    """

    return_line_items: typing.Optional[typing.List[OrderReturnLineItem]] = pydantic.Field(default=None)
    """
    A collection of line items that are being returned.
    """

    return_service_charges: typing.Optional[typing.List[OrderReturnServiceCharge]] = pydantic.Field(default=None)
    """
    A collection of service charges that are being returned.
    """

    return_taxes: typing.Optional[typing.List[OrderReturnTax]] = pydantic.Field(default=None)
    """
    A collection of references to taxes being returned for an order, including the total
    applied tax amount to be returned. The taxes must reference a top-level tax ID from the source
    order.
    """

    return_discounts: typing.Optional[typing.List[OrderReturnDiscount]] = pydantic.Field(default=None)
    """
    A collection of references to discounts being returned for an order, including the total
    applied discount amount to be returned. The discounts must reference a top-level discount ID
    from the source order.
    """

    return_tips: typing.Optional[typing.List[OrderReturnTip]] = pydantic.Field(default=None)
    """
    A collection of references to tips being returned for an order.
    """

    rounding_adjustment: typing.Optional[OrderRoundingAdjustment] = pydantic.Field(default=None)
    """
    A positive or negative rounding adjustment to the total value being returned. Adjustments are commonly
    used to apply cash rounding when the minimum unit of the account is smaller than the lowest
    physical denomination of the currency.
    """

    return_amounts: typing.Optional[OrderMoneyAmounts] = pydantic.Field(default=None)
    """
    An aggregate monetary value being returned by this return entry.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .inventory_count import InventoryCountParams
from .inventory_change import InventoryChangeParams


class BatchChangeInventoryResponseParams(typing_extensions.TypedDict):
    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    counts: typing_extensions.NotRequired[typing.Sequence[InventoryCountParams]]
    """
    The current counts for all objects referenced in the request.
    """

    changes: typing_extensions.NotRequired[typing.Sequence[InventoryChangeParams]]
    """
    Changes created for the request.
    """
